#include "gnc-helpers.h"

#include <g-wrap.h>
#include "Group.h"

SCM
gnc_timespec2timepair(Timespec t) {
  SCM secs;
  SCM nsecs;

  secs = gh_longlong2scm(t.tv_sec);
  nsecs = gh_long2scm(t.tv_nsec);
  return(gh_cons(secs, nsecs));
}

Timespec
gnc_timepair2timespec(SCM x) {
  Timespec result;
  result.tv_sec = gh_scm2longlong(gh_car(x));
  result.tv_nsec = gh_scm2long(gh_cdr(x));
  return(result);
}

int
gnc_timepair_p(SCM x) {
  return(gh_pair_p(x) && gh_exact_p(gh_car(x)) && gh_exact_p(gh_cdr(x)));
}

Timespec
gnc_transaction_get_date_posted(Transaction *t) {
  Timespec result;
  xaccTransGetDateTS(t, &result);
  return(result);
}

Timespec
gnc_transaction_get_date_entered(Transaction *t) {
  Timespec result;
  xaccTransGetDateEnteredTS(t, &result);
  return(result);
}

Timespec
gnc_split_get_date_reconciled(Split *s) {
  Timespec result;
  xaccSplitGetDateReconciledTS(s, &result);
  return(result);
}

void
gnc_transaction_set_date_posted(Transaction *t, const Timespec d) {
  xaccTransSetDateTS(t, &d);
}

void
gnc_transaction_set_date_entered(Transaction *t, const Timespec d) {
  xaccTransSetDateEnteredTS(t, &d);
}

static int
_gnc_scm_traversal_adapter_(Transaction *t, void *data) {
  SCM scm_trans = POINTER_TOKEN_to_SCM(make_POINTER_TOKEN("Transaction*", t));
  SCM thunk = *((SCM *) data);
  SCM result = gh_call1(thunk, scm_trans);
  if(result == SCM_BOOL_F) {
    return(GNC_F);
  } else {
    return(GNC_T);
  }
}

gncBoolean
gnc_scmGroupStagedTransactionTraversal(AccountGroup *grp,
                                       unsigned int new_marker,
                                       SCM thunk) {
  return((gncBoolean) xaccGroupStagedTransactionTraversal(grp, new_marker,
                                             _gnc_scm_traversal_adapter_,
                                             &thunk));
}

gncBoolean
gnc_scmAccountStagedTransactionTraversal(Account *a,
                                         unsigned int new_marker,
                                         SCM thunk) {
  return((gncBoolean) xaccAccountStagedTransactionTraversal(a, new_marker,
                                               _gnc_scm_traversal_adapter_,
                                               &thunk));
}
