/* -*-gnucash-c-*-
   gnucash.c -- gnucash startup function.
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
                                                                   
   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
                                                                   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
   (C) 1998 Rob Browning <rlb@cs.utexas.edu>
*/

#define _GNU_SOURCE
#include <stdio.h>

#include <guile/gh.h>

#include "FileDialog.h"
#include "top-level.h"
#include "util.h"
#include "gnucash.h"

static int guile_is_initialized = 0;
static int gargc;
static char **gargv;

/* ============================================================== */

void
gnc_set_double_entry_restriction(const int state) {

  if(state == 0) {
    gh_eval_str("(gnc:config-var-set! gnc:*double-entry-restriction* #f)");
  } else if(state == 1) {
    gh_eval_str("(gnc:config-var-set! gnc:*double-entry-restriction* 'force)");
  } else if(state == 2) {
    gh_eval_str("(gnc:config-var-set! gnc:*double-entry-restriction* 'collect)");
  } else {
    char *tmp = NULL;
    asprintf(&tmp, "(gnc:config-var-set! gnc:*double-entry-restriction* %d)", state);
    assert(tmp);
    gh_eval_str(tmp);
    free(tmp);
  }
}

/* ============================================================== */

/* Because g-wrap doesn't declare the prototype anywhere */
void scm_init_gnc_module();
/* Because swig doesn't declare the prototype anywhere */
/* void gnucash_swig_init(); */

static void 
gnucash_main_helper(int argc, char *argv[]) {
  
  char *bootstrap_scm = getenv("GNC_BOOTSTRAP_SCM");
  char *scm_load_path = getenv("GNC_SCM_LOAD_PATH");
  char *run_as_shell = getenv("GNC_RUN_AS_SHELL");

  guile_is_initialized = 1;

  init_gnc();
  /* scm_init_gnc_module(); */
  /* gnucash_swig_init(); */

  if(!bootstrap_scm) {
    if(setenv("GNC_BOOTSTRAP_SCM", GNC_BOOTSTRAP_SCM, 1) != 0) {
      fprintf(stderr, "gnucash: setenv(%s, %s, 1) failed.  Aborting.\n",
              "GNC_BOOTSTRAP_SCM", GNC_BOOTSTRAP_SCM);
      exit(1);
    }
    bootstrap_scm = GNC_BOOTSTRAP_SCM;
  }
  fprintf(stderr, "gnucash: bootstrap file is %s\n", bootstrap_scm);

  if(run_as_shell) {
    /* Add the "--" to keep gh_repl from interpreting the gnucash args */
    
    int scm_argc = argc + 1;
    char **scm_argv = malloc(scm_argc * sizeof(char *));
    assert(scm_argv);

    scm_argv[0] = (char *) argv[0];
    scm_argv[1] = "--";
    memcpy(&(scm_argv[2]), &(argv[1]), (argc - 1) * sizeof(char *));
    gh_repl(scm_argc, scm_argv);
    
  } else {

    SCM func;

    /* Load the bootstrap file. */
    if(gh_eval_file(bootstrap_scm) == SCM_BOOL_F) {
      fprintf(stderr, "gnucash: (load %s) failed\n", bootstrap_scm);
      exit(1);
    }

    /* Now finally load the startup file. */
    func = gh_eval_str("gnc:load");
    
    if(gh_procedure_p(func)) {
      SCM init_scm = gh_str02scm("startup.scm");
      gh_call1(func, init_scm);
    } else {
      fprintf(stderr, "gnucash: gnc:load doesn't exist.  Aborting.\n");
      exit(1);
    }
    
    /* and then run gnc:main. */
    func = gh_eval_str("gnc:main");
    
    if(gh_procedure_p(func)) {
      gh_call0(func);
    } else {
      fprintf(stderr, "gnucash: (gnc:main) doesn't exist.  Aborting.\n");
      exit(1);
    }
  }
  
  exit(0);
}

/* ============================================================== */

void
gnc_shutdown (int exit_status) {
  static int handled_shutdown = 0;

  if(guile_is_initialized) {
    /* This should be gh_lookup, but that appears to be broken */
    SCM scm_shutdown = gh_eval_str("gnc:shutdown");

    if(gh_procedure_p(scm_shutdown)) {
      SCM scm_exit_code = gh_long2scm(exit_status);

      gh_call1(scm_shutdown, scm_exit_code);
      /* this will never execute */
      handled_shutdown = 1;
    }
  } else {
    _gnc_shutdown_(exit_status);
    exit(exit_status);
  }
}

/* ============================================================== */

void
_gnc_shutdown_(int exit_status) {
  /* Put any C level routines that must be called at shutdown here
     gnc:shutdown will call this eventually...
  */

  /* These calls should probably eventually be moved to the scheme
     side... */
  gncFileQuerySave();
  /*XtUnmapWidget(gnc_get_ui_data());*/     /* make it disappear quickly */
  gncFileQuit();
  gnc_ui_shutdown();
}

/* ============================================================== */

int
main(int argc, char *argv[])
{
  gargc = argc;
  gargv = argv;

  gh_enter(argc, argv, gnucash_main_helper);
  /* This will never return. */
  
  assert(0);  /* Just to be anal */
  return 0;   /* Just to shut gcc up. */
}

/* ==================== END OF FILE ============================== */


